from pathlib import Path

from textual import on
from textual.containers import Horizontal, ScrollableContainer, Vertical
from textual.screen import ModalScreen
from textual.widgets import (
    Button,
    Checkbox,
    Collapsible,
    Input,
    Label,
    Log,
    RadioButton,
    RadioSet,
    Static,
)

from .file_screen import FilePickButton
from .server_cfg import ServerFunctions
from .utils import FilteredConfigTree


class ServerScreen(ModalScreen, ServerFunctions):
    CSS_PATH = "style.css"
    config_dict = {}

    BINDINGS = [
        ("escape", "app.pop_screen", "Back to main."),
        ("ctrl+s", "save_screen", "Save screenshot"),
    ]

    def action_save_screen(self):
        image_folder = Path("./images/")
        image_folder.mkdir(exist_ok=True)
        self.app.save_screenshot(image_folder.joinpath("server.svg"))

    def compose(self):
        with ScrollableContainer():
            yield Label("Create server")
            with Collapsible(title="Optional config", id="config"):
                with Horizontal():
                    with Vertical():
                        with Collapsible(title="Sinara server container name"):
                            yield Input(
                                value="jovyan-single-use", name="instanceName"
                            )

                        with Collapsible(title="Memory"):
                            yield Static(
                                "Amount of memory requested"
                                " for server container"
                            )
                            yield Input(value="4g", name="memRequest")

                            yield Static(
                                "Maximum amount of memory for server container"
                            )
                            yield Input(value="8g", name="memLimit")

                            yield Static(
                                "Maximum amount of shared memory for"
                                " server container"
                            )
                            yield Input(value="512m", name="shm_size")

                        with Collapsible(title="Cpu"):
                            yield Static(
                                " Number of CPU cores to use for"
                                " server container"
                            )
                            yield Input(
                                value="4", type="number", name="cpuLimit"
                            )

                        with Collapsible(
                            title="Folders Path (only used in basic mode)"
                        ):
                            yield Static("Path to data fodler on host")
                            yield FilePickButton(name="jovyanDataPath")

                            yield Static("Path to work fodler on host")
                            yield FilePickButton(name="jovyanWorkPath")

                            yield Static("Path to tmp fodler on host")
                            yield FilePickButton(name="jovyanTmpPath")

                    with Vertical():
                        with Collapsible(title="Server config"):
                            yield Checkbox(
                                "Create Folders",
                                value=True,
                                name="createFolders",
                            )
                            yield Checkbox(
                                "Gpu Enabled", value=True, name="gpuEnabled"
                            )
                            yield Checkbox(
                                "Run server without password protection",
                                value=False,
                                name="insecure",
                            )
                            yield Checkbox(
                                "Use expermiental server images",
                                value=False,
                                name="experimental",
                            )

                        with Collapsible(title="Server platform"):
                            yield Static("Host where the server is run")

                            with RadioSet(name="platform"):
                                yield RadioButton("Desktop", value=True)
                                yield RadioButton("Remote VM")

                        with Collapsible(title="Run Mode"):
                            yield Static(
                                "Quick - work, data, tmp will"
                                " be mounted inside docker volumes"
                            )
                            yield Static(
                                "Basic - work, data, tmp will"
                                " be mounted from host folders"
                            )

                            with RadioSet(name="runMode"):
                                yield RadioButton("Quick", value=True)
                                yield RadioButton("Basic")

            with Horizontal():
                with Vertical():
                    yield Static("Sinara image for:")
                    with RadioSet(name="sinara_image_num"):
                        yield RadioButton("CV", value=True)
                        yield RadioButton("ML")
                with Vertical():
                    yield Static("Ready configs [~/.sinaraX]:")
                    self.config_tree = FilteredConfigTree(
                        Path().home().joinpath(".sinaraX/"), id="ready_configs"
                    )
                    self.selected_config_path = ""
                    yield self.config_tree
                    with Horizontal():
                        yield Button(
                            "Load selected",
                            id="load_cfg_button",
                            classes="button",
                            variant="primary",
                        )
                        yield Button(
                            "Remove selected",
                            id="remove_cfg_button",
                            classes="button",
                            variant="error",
                        )

            with Horizontal():
                yield Button(
                    "Create!",
                    id="server_button",
                    classes="button",
                    variant="success",
                )
                yield Button(
                    "Remove!",
                    id="server_remove_button",
                    classes="button",
                    variant="error",
                )
                yield Button(
                    "HELP",
                    id="help_button",
                    classes="button",
                    variant="primary",
                )

                yield Button(
                    "Update image",
                    id="update_image_button",
                    classes="button",
                    variant="success",
                )

            with Horizontal():
                yield Button(
                    "Start!",
                    id="server_start_button",
                    classes="button",
                    variant="success",
                )
                yield Button(
                    "Stop!",
                    id="server_stop_button",
                    classes="button",
                    variant="warning",
                )
                yield Button(
                    "Get config",
                    id="get_cfg_button",
                    classes="button",
                    variant="success",
                )
                yield Button(
                    "Save config",
                    id="save_cfg_button",
                    classes="button",
                    variant="success",
                )

            with Horizontal():
                yield Button(
                    "< Back",
                    id="back_button",
                    classes="back_button button",
                    variant="primary",
                )
                yield Button(
                    "Exit",
                    id="exit_button",
                    classes="exit_button button",
                    variant="error",
                )

            yield Static()

            self.log_window: Log = Log(
                highlight=True, id="output_text_area", classes="log_window"
            )
            yield self.log_window

    @on(Button.Pressed, "#server_button")
    def server_button(self):
        if self.generate_config():
            cmd = "sinara server create --verbose "
            for key, val in self.config_dict.items():
                if len(val) > 1:
                    val = f"'{val}'"

                cmd += f"--{key} {val} "

            self.cmd(cmd)

    @on(Button.Pressed, "#server_remove_button")
    def server_remove_button(self):
        cmd = "sinara server remove"
        result = self.generate_config()
        if result:
            if self.config_dict.get("instanceName"):
                cmd += " --instanceName " + str(
                    self.config_dict.get("instanceName")
                )

        self.cmd(cmd)

    @on(Button.Pressed, "#server_stop_button")
    def server_stop_button(self):
        cmd = "sinara server stop"
        result = self.generate_config()
        if result:
            if self.config_dict.get("instanceName"):
                cmd += " --instanceName " + str(
                    self.config_dict.get("instanceName")
                )

        self.cmd(cmd)

    @on(Button.Pressed, "#server_start_button")
    def server_start_button(self):
        cmd = "sinara server start"
        result = self.generate_config()
        if result:
            if self.config_dict.get("instanceName"):
                cmd += " --instanceName " + str(
                    self.config_dict.get("instanceName")
                )

        self.cmd(cmd)

    @on(Button.Pressed, "#update_image_button")
    def update_image_button(self):
        cmd = "sinara server update"
        result = self.generate_config()
        if result:
            image = self.config_dict.get("image")
            if image:
                cmd += " --image "
                if "cv" in image:
                    cmd += "cv "
                else:
                    cmd += "ml "

                if "exp" in image:
                    cmd += "--experimental"
            else:
                cmd += " -h"

        self.cmd(cmd)

    @on(Button.Pressed, "#help_button")
    def help_button(self):
        self.cmd("sinara server create -h")

    @on(Button.Pressed, "#back_button")
    def back_button(self):
        self.dismiss()

    @on(Button.Pressed, "#exit_button")
    def exit_button(self):
        self.parent.exit()

    @on(FilteredConfigTree.FileSelected)
    def select_config(self, event: FilteredConfigTree.FileSelected):
        return super().select_config(event)

    @on(Button.Pressed, "#remove_cfg_button")
    def remove_config_file_button(self):
        return super().remove_config_file_button()

    @on(Button.Pressed, "#load_cfg_button")
    def load_cfg_button(self):
        return super().load_cfg_button()

    @on(Button.Pressed, "#get_cfg_button")
    def get_cfg_button(self):
        return super().get_cfg_button()

    @on(Button.Pressed, "#save_cfg_button")
    def save_cfg_button(self):
        return super().save_cfg_button()



from pathlib import Path
from typing import Iterable

from textual import on
from textual.containers import Horizontal, ScrollableContainer
from textual.screen import ModalScreen
from textual.widget import Widget
from textual.widgets import Button, DirectoryTree, Input, Label, Static


class FilteredDirectoryTree(DirectoryTree):
    def filter_paths(self, paths: Iterable[Path]) -> Iterable[Path]:
        return [
            path
            for path in paths
            if (path.is_dir() and (not path.name.startswith(".")))
        ]


class FileTreeScreen(ModalScreen[str | None]):
    DEFAULT_CSS = """
    """

    selected_path = None
    BINDINGS = [("escape", "app.pop_screen", "Back to main.")]

    @on(Button.Pressed, ".back_button")
    def back_button(self):
        self.dismiss(self.selected_path)

    @on(Button.Pressed, "#exit_button")
    def exit_button(self):
        self.parent.exit()

    @on(DirectoryTree.DirectorySelected)
    def select_record(self, event: DirectoryTree.DirectorySelected) -> None:
        self.selected_path = event.path.as_posix()
        self.static_path.update(self.selected_path)

    def compose(self):
        self.static_path = Static("[]")

        with Horizontal():
            yield Label("Selected path:")
            yield self.static_path

        with ScrollableContainer():
            with Horizontal():
                yield Button(
                    "OK",
                    id="ok_button",
                    classes="back_button button",
                    variant="success",
                )
                yield Button(
                    "< Back",
                    id="back_button",
                    classes="back_button button",
                    variant="primary",
                )
                yield Button(
                    "Exit",
                    id="exit_button",
                    classes="exit_button button",
                    variant="error",
                )
            yield FilteredDirectoryTree(Path().home().as_posix())


class FilePickButton(Widget, can_focus=True):
    DEFAULT_CSS = """
    FilePickButton{
        height: auto;
    }
    FilePickButton > Horizontal > .file-pick-label{
        height: auto;
        padding: 0 2;
        border: none;
        min-width: 5;
        max-width: 95%;
        width: 85%;
        margin-left: 1;
    }
    FilePickButton > Horizontal > .file-pick-button{
        min-width: 5;
        border: none;
        padding: 0;
        margin: 0;
        &:hover {
            border: none;
        }
        &.-active {
            border: none;
        }
    }
    FilePickButton > Horizontal{
        margin-bottom: 1;
    }
    """

    value = ""

    def compose(self):
        with Horizontal():
            self.input = Input(classes="file-pick-label")
            yield self.input
            yield Button("F", classes="file-pick-button", variant="primary")

    @on(Button.Pressed)
    def open_file_tree_screen(self):
        self.app.push_screen(FileTreeScreen(), self.update)

    def update(self, value: str):
        self.value = str(value)
        self.input.value = value

    @on(Input.Changed)
    def input_change(self, event: Input.Changed) -> None:
        self.value = event.value



from .config import FilteredConfigTree, generate_from_screen, load_from_file
from .process import start_cmd

__all__ = [
    "generate_from_screen",
    "load_from_file",
    "start_cmd",
    "FilteredConfigTree",
]



from pathlib import Path

from sinaraml._version import __version__ as sinaraml_version
from textual import on, work
from textual.app import App
from textual.containers import Horizontal
from textual.events import Event
from textual.widgets import Button, Label, Log, Static

try:
    from sinaraX._version import __version__ as sinaraX_version
except ImportError:
    sinaraX_version = "__dev__"

from .server import ServerScreen
from .server_cfg import BaseFunctions
from .update import UpdateScreen
from .utils.config import AppConfig
from .utils.infra import check_docker, check_docker_group, check_platform


class SinaraX(App, BaseFunctions):
    CSS_PATH = "style.css"
    system_info_data = None
    config = AppConfig()
    SCREENS = {
        "server_screen": ServerScreen(),
        "update_screen": UpdateScreen(),
    }

    BINDINGS = [
        ("escape", "exit", "Exit"),
        ("ctrl+s", "save_screen", "Save screenshot"),
    ]

    def action_exit(self):
        self.exit()

    def action_save_screen(self):
        image_folder = Path("./images/")
        image_folder.mkdir(exist_ok=True)
        self.save_screenshot(image_folder.joinpath("main.svg"))

    def compose(self):
        yield Label(
            f"SinaraX by MiXaiLL76; sinaraml=={sinaraml_version};"
            f" sinaraX=={sinaraX_version}"
        )

        with Horizontal():
            yield Button(
                "SERVER",
                id="create_server_button",
                classes="button",
                variant="primary",
            )

            yield Button(
                "UPDATE",
                id="cli_update_button",
                classes="button",
                variant="primary",
            )

            yield Button(
                "Exit",
                id="exit_button",
                classes="exit_button button",
                variant="warning",
            )

        yield Button(
            "Check system",
            id="check_button",
            classes="button",
            variant="primary",
        )

        yield Static()

        self.log_window: Log = Log(
            highlight=True, id="output_text_area", classes="log_window main_log"
        )
        yield self.log_window

        if self.system_info_data is None:
            self.get_system_info()

    @work(thread=True)
    def get_system_info(self):
        self.system_info_data = {
            "docker_info": check_docker(),
            "platform_info": check_platform(),
            "group_info": check_docker_group(),
        }
        lines = []
        if self.system_info_data["docker_info"]["ok"]:
            docker_version = self.system_info_data["docker_info"][
                "ServerVersion"
            ]
            lines.append(f"Docker is running : True ; {docker_version}")
            for _docker_err in self.system_info_data["docker_info"][
                "ServerErrors"
            ]:
                lines.append(_docker_err)

        else:
            lines.append("Docker is running : False")

        if self.system_info_data["group_info"]["username"]:
            _USER = self.system_info_data["group_info"]["username"]
            lines.append(f"System user : {_USER}")

            docker_group = self.system_info_data["group_info"]["docker_group"]
            lines.append(f"Docker group created : {docker_group}")

            user_in_group = self.system_info_data["group_info"]["user_in_group"]
            lines.append(f"{_USER} in group docker : {user_in_group}")

        _platform = self.system_info_data["platform_info"]["platform"]

        if self.system_info_data["platform_info"]["ok"]:
            lines.append(f"Platform : True ; {_platform}")
        else:
            lines.append(f"Platform : False ; {_platform} USE WSL or Linux!")

        max_index = -1
        for line in lines:
            _idx = line.find(":")
            if _idx > max_index:
                max_index = _idx

        for i, line in enumerate(lines):
            _idx = line.find(":")
            lines[i] = line[:_idx] + " " * (max_index - _idx) + line[_idx:]

        self.write_log_lines(lines)

    @on(Button.Pressed, "#check_button")
    def check_button(self):
        self.system_info_data = None
        self.get_system_info()

    @on(Button.Pressed, "#create_server_button")
    def create_server_button(self):
        self.push_screen("server_screen")

    @on(Button.Pressed, "#cli_update_button")
    def cli_update_button(self):
        self.push_screen("update_screen")

    @on(Button.Pressed, "#exit_button")
    def exit_button(self):
        self.exit()

    def on_event(self, event: Event):
        self.app.refresh(layout=True)
        return super().on_event(event)



import json
import os
from pathlib import Path

from textual import work

from .utils import (
    FilteredConfigTree,
    generate_from_screen,
    load_from_file,
    start_cmd,
)


class BaseFunctions:
    def write_log_lines(self, lines):
        self.log_window.clear()

        if type(lines) is list:
            self.log_window.write_lines(lines)

        elif type(lines) is str:
            self.log_window.write(lines)

        else:
            self.log_window.write(f"{type(lines)=} not supported!")

        self.app.refresh(layout=True)

    @work(thread=True)
    def cmd(self, cmd: str):
        self.write_log_lines([cmd + "\n"])
        lines = [" "]
        for decoded_line in start_cmd(cmd):
            if len(decoded_line) > 0:
                if lines[-1][-1] == "\r":
                    lines[-1] = decoded_line
                else:
                    lines.append(decoded_line)

            self.write_log_lines([cmd + "\n"] + lines[1:])


class ServerFunctions(BaseFunctions):
    def generate_config(self):
        result = generate_from_screen(self)
        return result

    @work(thread=True)
    def reload_config_dir(self):
        sinaraX_path = Path().home().joinpath(".sinaraX")
        sinaraX_path.mkdir(exist_ok=True)
        self.config_tree.path = sinaraX_path

    @work(thread=True)
    def save_cfg(self):
        result = self.generate_config()
        if result:
            sinaraX_path = Path().home().joinpath(".sinaraX")
            sinaraX_path.mkdir(exist_ok=True)
            config_name = sinaraX_path.joinpath(
                self.config_dict.get("instanceName", "base.") + ".json"
            )
            with open(config_name, "w") as fd:
                json.dump(self.config_dict, fd)

            self.log_window.write_line(f"Config {config_name} saved!")
            self.reload_config_dir()
        else:
            self.log_window.write_line("Config not saved!")

    def select_config(self, event: FilteredConfigTree.FileSelected):
        self.selected_config_path = event.path.as_posix()

    def remove_config_file_button(self):
        selected_file = Path(str(self.selected_config_path))
        if selected_file.is_file():
            os.remove(selected_file.as_posix())
            self.log_window.write_line(f"Config {selected_file} removed!")
        self.reload_config_dir()

    def load_cfg_button(self):
        selected_file = Path(str(self.selected_config_path))
        if selected_file.is_file():
            load_from_file(self, selected_file.as_posix())
            self.log_window.write_line(f"Config {selected_file} loaded!")
        self.reload_config_dir()

    def get_cfg_button(self):
        self.log_window.clear()
        result = self.generate_config()
        if result:
            self.write_log_lines(json.dumps(self.config_dict, indent=4))

    def save_cfg_button(self):
        self.save_cfg()



import getpass
import json
import platform

from .process import start_cmd

check_docker_cmd = "docker info -f json"


def check_docker():
    lines = []
    failed = False
    for line in start_cmd(check_docker_cmd):
        lines.append(line)
        if "failed" in line:
            failed = True

        if "docker: not found" in line:
            failed = True

    result = {
        "ok": not failed,
        "ServerVersion": None,
        "NCPU": None,
        "MemTotal": None,
        "ServerErrors": [],
    }

    if not failed:
        output = "".join(lines)
        start_json = output.find("{")
        end_json = len(output) - output[::-1].find("}")
        if (start_json != -1) and (end_json != -1):
            docker_info = json.loads(output[start_json:end_json])
            result["NCPU"] = docker_info.get("NCPU")
            result["ServerErrors"] = docker_info.get("ServerErrors", [])
            result["ServerVersion"] = docker_info.get("ServerVersion")
            result["MemTotal"] = round(
                int(docker_info.get("MemTotal")) / 1024 / 1024, 2
            )
            for err in result.get("ServerErrors"):
                if "error during connect" in err:
                    result["ok"] = False
    return result


def check_platform():
    _platform = platform.system()
    is_windows = "Windows" in _platform
    result = {
        "ok": not is_windows,
        "platform": _platform,
    }
    return result


def check_docker_group():
    result = {
        "docker_group": False,
        "user_in_group": False,
        "is_unix": False,
        "username": getpass.getuser(),
    }

    try:
        import grp

        docker_group = grp.getgrnam("docker")
        result["docker_group"] = True
        result["is_unix"] = True
        result["user_in_group"] = result["username"] in docker_group.gr_mem
    except ModuleNotFoundError:
        pass
    except KeyError:
        pass

    return result
